﻿using BizTracker.Library;
using BizTracker.Library.Helpers;
using BizTracker2015.WebApp.Helpers;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using workingIT.Library;

namespace BizTracker2015.WebApp.Controllers
{
  [OutputCacheAttribute(VaryByParam = "*", Duration = 0, NoStore = true)] // will be applied to all actions in MyController, unless those
  public class CommonController : Controller
  {

    private string TRACKING_SERVERNAME = GeneralHelper.GetTrackingServer();
    private string MANAGEMENT_SERVERNAME = GeneralHelper.GetMgmtServer();
    private string TRACKING_DBNAME = GeneralHelper.GetTrackingDb();
    private string MANAGEMENT_DBNAME = GeneralHelper.GetMgmtDb();

    public BizTrackerObject GetBTObject
    {

      get
      {
        //In unitTest (needs mockup HttpContext, but not important right now)
        //I have tested manually in Real web context in debug mode, and the else
        //Is indeed hit on the second request.
        if (!GeneralHelper.ValidLicenseKey())
        {
          throw (new ApplicationException(String.Format("Not a valid license key.. {0}", CommonHelper.GetKey("licenseKey", ""))));
        }

        if (System.Web.HttpContext.Current == null)
          return new BizTrackerObject(TRACKING_SERVERNAME, MANAGEMENT_SERVERNAME, TRACKING_DBNAME, MANAGEMENT_DBNAME);

        if (System.Web.HttpContext.Current.Session["bto"] == null)
        {
          BizTrackerObject bto = new BizTrackerObject(TRACKING_SERVERNAME, MANAGEMENT_SERVERNAME, TRACKING_DBNAME, MANAGEMENT_DBNAME);
          System.Web.HttpContext.Current.Session["bto"] = bto;
          return bto;
        }
        else
        {
          return (BizTrackerObject)System.Web.HttpContext.Current.Session["bto"];
        }
      }

    }

    [HttpGet]
    public JsonResult GetCustomProperties()
    {
      var model = GeneralHelper.GetCustomProperties();
      return Json(model, JsonRequestBehavior.AllowGet);
    }

    [HttpGet]
    public JsonResult GetParameters()
    {
      var advancedSearch = GeneralHelper.AdvancedSearch();

      var outputModel = new {advancedSearch = advancedSearch};
      return Json(outputModel,JsonRequestBehavior.AllowGet);
    }

    [HttpGet]
    public JsonResult IndentByMessageID(string messageID)
    {

      long i;
      bool ii;
      var text = GetBTObject.GetMessageBody(messageID, out ii, out i, 0);
      string output = text;

      if (XmlHelper.IsXML(text))
      {
        output = XmlHelper.IndentXML(text);
        // output = CommonHelper.GenerateHTMLCRTL(output);
      }
      else if (XmlHelper.IsEDI(text))
      {
        output = XmlHelper.EDIIndent(text);

      }
      return Json(output, JsonRequestBehavior.AllowGet);

    }

    [HttpGet]
    public JsonResult GetRawSettingData()
    {
      //var model = DataStorageHelper.GetValue("Morten");
      var model = "Ander";
      return Json(model, JsonRequestBehavior.AllowGet);
    }

    [HttpGet]
    [ValidateInput(false)]
    public JsonResult Indent(string text)
    {
      string output = text;
      if (XmlHelper.IsXML(text))
      {
        output = XmlHelper.IndentXML(text);
       // output = CommonHelper.GenerateHTMLCRTL(output);
      }
      else if(XmlHelper.IsEDI(text))
      {
        output = XmlHelper.EDIIndent(text);

      }
      return Json(output, JsonRequestBehavior.AllowGet);

    }
  }
}