﻿using BizTracker.Library;
using BizTracker.Library.Helpers;
using BizTracker.Library.Models;
using BizTracker2015.WebApp.Helpers;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web;

using System.Web.Mvc;
using workingIT.Library;

namespace BizTracker2015.WebApp.Controllers
{
  [OutputCacheAttribute(VaryByParam = "*", Duration = 0, NoStore = true)] // will be applied to all actions in MyController, unless those
  public class TrackController : Controller
  {
    private string TRACKING_SERVERNAME = GeneralHelper.GetTrackingServer();
    private string MANAGEMENT_SERVERNAME = GeneralHelper.GetMgmtServer();
    private string TRACKING_DBNAME = GeneralHelper.GetTrackingDb();
    private string MANAGEMENT_DBNAME = GeneralHelper.GetMgmtDb();



    private HttpResponseMessage ThrowHttpError(string reason, string content)
    {
      var resp = new HttpResponseMessage(HttpStatusCode.NotFound)
      {
        Content = new StringContent(content),
        ReasonPhrase = reason
      };
      throw new System.Web.Http.HttpResponseException(resp);

    }

    private void setServerName(string serverName)
    {
      System.Web.HttpContext.Current.Session["server"] = serverName;
      System.Web.HttpContext.Current.Session["bto"] = null;

    }

    private void clearServerName()
    {
      System.Web.HttpContext.Current.Session["server"] = null;
      System.Web.HttpContext.Current.Session["bto"] = null;
    }

    //Either from TRACKING_SERVERNAME or Session (if available)
    private string currentServer()
    {
      if (System.Web.HttpContext.Current == null)
      {
        return TRACKING_SERVERNAME;
      }
      else
      {
        if (System.Web.HttpContext.Current.Session["server"] == null)
        {
          return TRACKING_SERVERNAME;
        }
        else
        {
          string s = System.Web.HttpContext.Current.Session["server"].ToString();
          if (String.IsNullOrEmpty(s))
            return TRACKING_SERVERNAME;
          else
            return s;
        }
      }
    }

    [HttpGet]
    public ActionResult Servers()
    {
      var model = GeneralHelper.GetServers();
      return View(model);
    }

    [HttpPost]
    public ActionResult Servers(string server)
    {
      setServerName("Morten");
      return RedirectToAction("Index", "Home");
    }

    [HttpPost]
    public void SetServerName(string serverName)
    {
      setServerName(serverName);
    }
    [HttpPost]
    public void ResetServer()
    {
      clearServerName();
    }

    [HttpGet]
    public JsonResult GetServerNames()
    {
      return Json(GeneralHelper.GetServers(), JsonRequestBehavior.AllowGet);
    }


    [HttpGet]
    public JsonResult GetCurrentServerName()
    {
      return Json(currentServer(), JsonRequestBehavior.AllowGet);
    }

    public BizTrackerObject GetBTObject
    {

      get
      {
        //In unitTest (needs mockup HttpContext, but not important right now)
        //I have tested manually in Real web context in debug mode, and the else
        //Is indeed hit on the second request.
        if (!GeneralHelper.ValidLicenseKey())
        {
          throw (new ApplicationException(String.Format("Not a valid license key.. {0}", CommonHelper.GetKey("licenseKey", ""))));
        }

        if (System.Web.HttpContext.Current == null)
          return new BizTrackerObject(currentServer(), currentServer(), TRACKING_DBNAME, MANAGEMENT_DBNAME);

        if (System.Web.HttpContext.Current.Session["bto"] == null)
        {
          BizTrackerObject bto = new BizTrackerObject(currentServer(), currentServer(), TRACKING_DBNAME, MANAGEMENT_DBNAME);
          System.Web.HttpContext.Current.Session["bto"] = bto;
          return bto;
        }
        else
        {
          return (BizTrackerObject)System.Web.HttpContext.Current.Session["bto"];
        }
      }

    }

    [HttpGet]
    public JsonResult GetEditionText()
    {
      //TODO cache
      var model = GeneralHelper.GetLicenseKeyString();
      return Json(model, JsonRequestBehavior.AllowGet);
    }

    [HttpGet]
    [OutputCache(Duration = 180)]
    public JsonResult GetPortAlerts(bool expiredOnly)
    {
      var rawString = GeneralHelper.GetPortAlerts();
      if (rawString.Count() == 0)
      {
        List<PortData> emptyModel = new List<PortData>();
        return Json(emptyModel, JsonRequestBehavior.AllowGet);
      }
      //TODO Pass datetime to limit search??
      var model = GetBTObject.GetPortData();
      if (expiredOnly)
        model = model.Where(a => a.Expired).ToList();
      return Json(model, JsonRequestBehavior.AllowGet);

    }


    [HttpGet]
    [OutputCache(Duration = 180)]
    public JsonResult GetAlerts(bool expiredOnly)
    {
      var rawString = GeneralHelper.GetMessageTypeAlerts();
      if (rawString.Count() == 0)
      {
        List<MessageTypeData> emptyModel = new List<MessageTypeData>();
        return Json(emptyModel, JsonRequestBehavior.AllowGet);
      }
      //TODO Pass datetime to limit search??
      var model = GetBTObject.GetMessageTypeData();
      if (expiredOnly)
        model = model.Where(a => a.Expired).ToList();
      return Json(model, JsonRequestBehavior.AllowGet);

    }


    [HttpGet]
    public JsonResult GetInstance(string instanceId,
      string activityId,
      string[] customProperties = null,
      bool manipulate = true

      )
    {
      var rawmodel = GetBTObject.GetDbInstance(instanceId, activityId,
        customProperties: (customProperties == null) ? null : customProperties.ToList());
      List<DbInstance> model;

      if (manipulate)
      {
        model = GetBTObject.ManipulateInstanceList(rawmodel.ToList());

      }
      else
      {
        model = rawmodel.ToList();
      }
      return Json(model, JsonRequestBehavior.AllowGet);
    }



    [HttpGet]
    public JsonResult GetInstances(int top,
      DateTime dtStart,
      DateTime dtStop,
      bool allDates,
      string portName,
      string messageType,
      string[] customProperties = null,
      bool errorOnly = false,
      bool manipulate = true,
      bool receiveOnly = true,
      string fromTime = null,
      string toTime = null)
    {
      try
      {
        bool wholeDays = true;
        var dtStart2 = dtStart.ToUniversalTime();
        var dtStop2 = dtStop.ToUniversalTime();

        if (wholeDays)
        {
          string sFromDate = dtStart2.ToString("yyyy-MM-dd") + " " + fromTime ?? "00:00:00";
          string sStopDate = dtStop2.ToString("yyyy-MM-dd") + " " + toTime ?? "23:59:59";

          dtStart2 = Convert.ToDateTime(sFromDate);
          dtStop2 = Convert.ToDateTime(sStopDate);

          dtStart2 = dtStart2.ToUniversalTime();
          dtStop2 = dtStop2.ToUniversalTime();
        }

        if (allDates)
        {
          dtStart2 = Convert.ToDateTime("1900-01-01");
          dtStop2 = Convert.ToDateTime("2900-01-01");
        }

        var rawmodel = GetBTObject.GetDbInstanceList(
              top: top,
              fromDate: dtStart2,
              toDate: dtStop2,
              receiveOnly: receiveOnly,
              portName: portName,
              messageType: messageType,
              errorOnly: errorOnly,
              customProperties: (customProperties == null) ? null : customProperties.ToList());

        List<DbInstance> model;
        if (manipulate)
        {
          model = GetBTObject.ManipulateInstanceList(rawmodel.ToList());

        }
        else
        {
          model = rawmodel.ToList();
        }
        return Json(model, JsonRequestBehavior.AllowGet);

      }
      catch (Exception ex)
      {
        Response.StatusCode = (int)System.Net.HttpStatusCode.BadRequest;
    //Log your exception
        return Json(new { Message = ex.Message },JsonRequestBehavior.AllowGet);
      }
    }

    [HttpGet]
    public JsonResult GetMessageBody(string messageID, bool all = false)
    {
      string model = null;
      bool fullSize;
      long fullSizeNumber;
      if (all)
      {
        model = GetBTObject.GetMessageBody(messageID, out fullSize, out fullSizeNumber);
      }
      else
      {
        //int messageSize = DataStorageHelper.GetMaxBodyLength();
        int messageSize = GeneralHelper.GetMaxBodyLength();
        if (messageSize == 0)
        {
          //Get all
          model = GetBTObject.GetMessageBody(messageID, out fullSize, out fullSizeNumber);
        }
        else
        {
          model = GetBTObject.GetMessageBody(messageID, out fullSize, out fullSizeNumber, messageSize);
        }
      }
      bool isindentable = XmlHelper.Indentable(model);
      var extendedModel = new { Body = model, FullSize = fullSize, fullByteSize = fullSizeNumber, size = model.Length, isIndentable = isindentable };
      var output =  Json(extendedModel, JsonRequestBehavior.AllowGet);
      output.MaxJsonLength = int.MaxValue;
      return output;
    }

    public FileResult SaveMessage2(string messageID)
    {
      string fileNameAndPath = String.Format("{0}.txt", messageID);
      HttpContext.Response.ClearHeaders();
      HttpContext.Response.AddHeader("content-disposition", "attachment; filename=biz.txt");

      Stream s = GetBTObject.GetMessageStream(messageID);
      var fileName = String.Format("C:\\temp\\{0}.txt", messageID);
      return File(s, System.Net.Mime.MediaTypeNames.Application.Octet, fileName);
    }

    [HttpGet]
    public JsonResult SaveMessage(string id, bool asXML = false)
    {
      string messageID = id;
      string fileNameAndPath = String.Format("{0}.{1}", messageID, (asXML) ? "xml" : "txt");
      //HttpContext.Response.ClearHeaders();
      HttpContext.Response.AddHeader("content-disposition", "attachment; filename=" + fileNameAndPath);
      HttpContext.Response.ContentType = "application/octet-stream";
      var m = GetBTObject.GetMessageStream(messageID);
      m.CopyTo(HttpContext.Response.OutputStream);
      return null;

    }

    [HttpGet]
    public JsonResult SaveMessageXML(string id)
    {
      return SaveMessage(id, true);
    }

    [HttpGet]
    public JsonResult GetMessageContext(string messageID)
    {
      var model = GetBTObject.GetMessageContext(messageID).OrderByDescending(c => c.Promoted);
      return Json(model, JsonRequestBehavior.AllowGet);
    }

  }
}